import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class JacobiCanvas extends ScaleCanvas implements KeyListener,MouseListener, MouseMotionListener {
    Manager M;
    Path2D.Double FIELD;
    EnhancedSlider PARAMETER,PITCH,YAW;
    EnhancedSlider[] COORD=new EnhancedSlider[3];
    Point JX;
    boolean drag;
    ListenSquare CONTROL;

     public JacobiCanvas() {
	 addMouseListener(this);
	 addMouseMotionListener(this);
	 addKeyListener(this);
	 CONTROL=new ListenSquare(0,0,600,150);CONTROL.on=1;
	 PARAMETER=new EnhancedSlider(0,0,530,27,200,new Color(0,100,0),"Z parameter");
	 YAW=new EnhancedSlider(0,30,530,27,265,Color.blue,"yaw");
	 PITCH=new EnhancedSlider(0,60,530,27,265,new Color(0,200,0),"pitch");
	 setScales(300,300,100);
     }
    
   public void paint(Graphics g2) {
      Graphics2D g=(Graphics2D) g2;
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      drawBG(g);
      drawControls(g);
      drawField(g);
   }

    public void drawBG(Graphics2D g) {
	g.setColor(new Color(20,30,40));
        g.fillRect(0,0,getWidth(),getHeight()); 
	Path2D.Double gp=new Path2D.Double();
	gp.moveTo(-10,0);
	gp.lineTo(10,0);
	gp.moveTo(0,-10);
	gp.lineTo(0,10);
	gp.moveTo(10,-10);
	gp.lineTo(-10,10);
	gp.moveTo(10,10);
	gp.lineTo(-10,-10);
	gp=transform(gp);
	g.setColor(Color.red);
	g.draw(gp);
    }

    public void drawControls(Graphics2D g) {
	PARAMETER.render(g);
	YAW.render(g);
	PITCH.render(g);
    }

    public void drawField(Graphics2D g) {
	if(FIELD!=null) {
	Path2D.Double p=transform(FIELD);
	   g.setColor(Color.white);
	   g.draw(p);
	}
    }

    public void computeJacobi() {
	FIELD=new Path2D.Double();
	double t=PARAMETER.getValue();
	double step=M.C.STEP.getValue();
	Vector[] V=JacobiComputer.jacobi(step,step,t);
	for(int i=0;i<V.length;++i) {
	    double[] A=view(V[i]);
	    if(i==0) FIELD.moveTo(A[0],A[1]);
	    if(i!=0) FIELD.lineTo(A[0],A[1]);
	}
    }

    public double[] view(Vector V) {
	Vector CENTER=new Vector(0,0,0);
	Vector W=pitch(CENTER,V);
	W=yaw(CENTER,W);
	double[] A={W.x[0],W.x[2],W.x[1]};
	return A;
    }


    public Vector pitch(Vector C,Vector V0) {
  	  double t=PITCH.getValue();
	  t=2*Math.PI*t;
	  Vector V=Vector.minus(V0,C);
	  V=Vector.pitch(t,V);
	  V=Vector.plus(V,C);
	  return V;
    }

    public Vector yaw(Vector C,Vector V0) {
	double t=YAW.getValue();
	t=2*Math.PI*t;	
        Vector V=Vector.minus(V0,C);
	V=Vector.yaw(t,V);
	V=Vector.plus(V,C);
        return V;

    }


        
    public void mousePressed(MouseEvent e) {
	MouseData J=MouseData.process(e);
	PARAMETER.activate(J.X);
	PITCH.activate(J.X);
	YAW.activate(J.X);
	drag=true;
    }

    
    public void mouseClicked(MouseEvent e) { 
	MouseData J=MouseData.process(e);
	doMouseClick(J.mode);
    }

    public void doMouseClick(int mode) {
	if(CONTROL.inside(JX)==1) {
	  PARAMETER.configure(JX);
	  PITCH.configure(JX);
	  YAW.configure(JX);
	  computeJacobi();
	  repaint();
	  return;
	}
        if(mode==1)  scaleUp(JX,0);
        if(mode==3)  scaleUp(JX,1);
	repaint();
    }


     public void mouseReleased(MouseEvent e) {
	 drag=false;	 
     }

     public void mouseEntered(MouseEvent e) {
	 requestFocus();
     }
     public void mouseExited(MouseEvent e) {}   

     public void mouseMoved(MouseEvent e) {
	MouseData J=MouseData.process(e);
	if(drag==true) {
	   PARAMETER.configure(J.X);
	   PITCH.configure(J.X);
	   YAW.configure(J.X);
	   computeJacobi();
	}
	JX=J.X;
     }

     public void mouseDragged(MouseEvent e) {
	MouseData J=MouseData.process(e);
	PARAMETER.configure(J.X);
	PITCH.configure(J.X);
	YAW.configure(J.X);
	computeJacobi();
	M.repaint();
     }
  public void keyTyped(KeyEvent e) {

      char ch=e.getKeyChar();
         M.repaint(); 
         if(ch=='z') doMouseClick(1);
         if(ch=='c') doMouseClick(3);
     }
    public void keyPressed(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {
    }

}

