import java.awt.event.*;
import java.awt.*;
import java.math.*;
import java.awt.geom.*;


public class GeodesicMaker {


    /**Here is the main routine.*/
    
    public static Vector[] geodesic(double step,double length,Vector U) {
	int lim=(int)(length/step);
	Vector[] V1=new Vector[lim];
	Vector[] V2=new Vector[lim];
	V1[0]=new Vector(U);
	V2[0]=new Vector(0,0,0);
	double f=U.x[0]*U.x[1];
	for(int i=1;i<lim;++i) {
	    Vector TANGENT=GroupStructure.structureField(V1[i-1]);
	    Vector dW1=TANGENT.scale(step);
	    V1[i]=Vector.plus(V1[i-1],dW1);
	    V1[i]=V1[i].unit();
	    Vector dW2=V1[i].scale(step);
	    V2[i]=GroupStructure.leftTranslation(V2[i-1],dW2);
	}
	return V2;
    }

    /**This gets the perfect geodesic with the initial unit vector*/
    
    public static Vector[] perfectGeodesic(double step,Vector U) {
	double length=EllipticIntegrals.period(U.x[0]*U.x[1]);
	return geodesic(step,length,U);
    }
    
    /**This gets the endpoint of the geodesic with the given length and initial direction*/
    
    public static Vector geodesicEndpoint(double step,double length,Vector U) {
	int lim=(int)(length/step);
	Vector V1;
	Vector V2;
	V1=new Vector(U);
	V2=new Vector(0,0,0);
	for(int i=1;i<lim;++i) {
	    Vector TANGENT=GroupStructure.structureField(V1);
	    Vector dW1=TANGENT.scale(step);
	    V1=Vector.plus(V1,dW1);
	    V1=V1.unit();
	    Vector dW2=V1.scale(step);
	    V2=GroupStructure.leftTranslation(V2,dW2);
	}
	return V2;
    }

    /**This gets the geodesic endpoint for a vector V which is not necessarily a unit vector*/
    public static Vector geodesicEndpoint(double step,Vector V) {
	Vector U=V.unit();
	double r=V.norm();
	return geodesicEndpoint(step,r,U);
    }
    /**END OF GEODESIC ROUTINES*/




    public static Vector[] flow(double step,double length,Vector U) {
	int lim=(int)(length/step);
	Vector[] LIST=new Vector[lim];
	LIST[0]=new Vector(U);
	for(int i=1;i<lim;++i) {
	    Vector TANGENT=GroupStructure.structureField(LIST[i-1]);
	    Vector dW1=TANGENT.scale(step);
	    LIST[i]=Vector.plus(LIST[i-1],dW1);
	    LIST[i]=LIST[i].unit();
	}
	return LIST;
    }

    public static Vector flowEndpoint(double step,double length,Vector U) {
	int lim=(int)(length/step);
	Vector V1;
	V1=new Vector(U);
	for(int i=1;i<lim;++i) {
	    Vector TANGENT=GroupStructure.structureField(V1);
	    Vector dW1=TANGENT.scale(step);
	    V1=Vector.plus(V1,dW1);
	    V1=V1.unit();
	}
	return V1;
    }


    public static Vector flowEndpointBackwards(double step,double length,Vector U) {
	int lim=(int)(length/step);
	Vector V1;
	V1=new Vector(U);
	for(int i=1;i<lim;++i) {
	    Vector TANGENT=GroupStructure.structureField(V1);
	    Vector dW1=TANGENT.scale(-step);
	    V1=Vector.plus(V1,dW1);
	    V1=V1.unit();
	}
	return V1;
    }


    

    
    
    /**GETTING VARIOUS INFO ABOUT THE GEODESIC*/
        
    public static double[] periodData(double step,Vector U) {
	double length=EllipticIntegrals.period(U.x[0]*U.x[1]);

	if(length>15) {
	    double[] data={0,0,0,0,0,0};
	    return data;
	}
	Vector V=geodesicEndpoint(step,length,U);
	double x=V.x[0];
	double y=V.x[1];
	double z=V.x[2];
	double invt=Math.sqrt(x*y);
	double[] data={length,x,y,z,invt,x/y};
	return data;
    }


    

    /**This gets the endpoint of the symmetric geodesic:  U.x[2] should be positive.
       Our drawing program does not use this routine, though it gives the same answer
       as the somewhat slicker routine we do use*/
    
    
    public static Vector geodesicEndpointSymm(double step,Vector U) {
	Vector V1=new Vector(U);
	Vector V2=new Vector(0,0,0);
        Vector V3=new Vector(V1);
        V3.x[2]=-V3.x[2];
        boolean test=false;
        int count=0;
        while(test==false) {
	    Vector TANGENT=GroupStructure.structureField(V1);
	    Vector dW1=TANGENT.scale(step);
	    V1=Vector.plus(V1,dW1);
	    V1=V1.unit();
	    Vector dW2=V1.scale(step);
	    V2=GroupStructure.leftTranslation(V2,dW2);
            if((Vector.dist(V1,V3)<.1)&&(V1.x[2]<V3.x[2])) test=true;
            ++count;
	}
	return V2;
    }

    
    /**PROJECTING THE GEODESIC INTO THE PLANE.
       These routines are used in the geodesic checker window*/
    
    /**this controls the projection.*/

    public static double[] turnProjection(double t,double[] data) {
	double[] data2={data[0],0,0,1};
	double c=Math.cos(t);
	double s=Math.sin(t);
	data2[1]=+c*data[1]+s*data[2];
	data2[2]=-s*data[1]+c*data[2];
	return data2;
    }
    

    /**The vector U should be a unit vector*/

    public static Path2D.Double elevationGeodesic(double proj,double step,double length,Vector U,boolean fit,boolean turns) {
	double[] data=periodData(step,U);
	double length2=length;
	data=turnProjection(proj,data);
	if(turns==true) length2=data[0]*length;
	Vector[] V=geodesic(step,length2,U);
	double[][] g=GeodesicAnalyzer.elevationGeodesic(data[1],data[2],V);
	if(fit==true) g=GeodesicAnalyzer.fitInSquare(g);
	Path2D.Double gp=new Path2D.Double();
	for(int i=0;i<V.length;++i) {
	    if(i==0) gp.moveTo(g[i][0],g[i][1]);
	    if(i!=0) gp.lineTo(g[i][0],g[i][1]);
	}
	return gp;
    }
    



}
