import java.awt.event.*;
import java.awt.*;
import java.math.*;
import java.awt.geom.*;


public class JacobiComputer {


    public static double findSwitch(double step,double crit,double R,double r) {
	double theta1=Math.random();
	double theta2=theta1+Math.PI;
	double theta3=0;
	
	for(int i=0;i<20;++i) {
	    theta3=(theta1+theta2)/2;
	    int t1=detTest(step,crit,R,r,theta1);
	    int t2=detTest(step,crit,R,r,theta2);
	    int t3=detTest(step,crit,R,r,theta3);
	    if(t1*t3==1) theta1=theta3;
	    if(t2*t3==1) theta2=theta3;
	}
	return theta3;
    }

    
    /**
       step is the integration stepsize
       crit is the period value of the critical loop level set
       R is the radius of the sphere of interest
       (x,y) is the point on the vector selection window
    **/
    

    public static int detTest(double step,double crit, double R,double r,double theta) {
	double x=r*Math.cos(theta);
	double y=r*Math.sin(theta);
	Vector V=Vector.chartToSphere(x,y);
	return detTest(step,R,V);
    }

    

    public static int detTest(double step,double R,Vector V) {
	Vector W1=JacobiComputer.jacobi1(step,step,R,V);
	Vector W2=JacobiComputer.jacobi2(step,step,R,V);
	double det=W1.x[0]*W2.x[1]-W1.x[1]*W2.x[0];
	if(det<0) return -1;
	return 1;
    }
    
    public static Vector jacobi1(double step1,double step2,double R,Vector U0) {
	Vector W=GroupStructure.structureField(U0);
	W=W.scale(step2); //along direction
	Vector U1=Vector.plus(U0,W);
	U1=U1.unit();
        Vector E0=GeodesicMaker.geodesicEndpoint(step1,R,U0);
        Vector E1=GeodesicMaker.geodesicEndpoint(step1,R,U1);
	Vector dE=Vector.minus(E1,E0);
	dE=dE.scale(1/step2);
	dE.x[0]=dE.x[0]*Math.exp(-E0.x[2]);
	dE.x[1]=dE.x[1]*Math.exp(+E0.x[2]);
	return dE;
    }
    
    
    public static Vector jacobi2(double step1,double step2,double R,Vector U0) {
	Vector V=GroupStructure.structureField(U0);
	Vector W=Vector.cross(U0,V);
	W=W.scale(step2); //transverse direction;
	Vector U1=Vector.plus(U0,W);
	U1=U1.unit();
        Vector E0=GeodesicMaker.geodesicEndpoint(step1,R,U0);
        Vector E1=GeodesicMaker.geodesicEndpoint(step1,R,U1);
	Vector dE=Vector.minus(E1,E0);
	dE=dE.scale(1/step2);
	dE.x[0]=dE.x[0]*Math.exp(-E0.x[2]);
	dE.x[1]=dE.x[1]*Math.exp(+E0.x[2]);
	return dE;
    }


    public static double criticalValue(double length) {
	double val0=EllipticIntegrals.periodInverse(length); //critical level set F-value
	val0=Math.sqrt(val0);
	Vector CRIT=new Vector(val0,val0,Math.sqrt(1-2*val0));
	if(Double.isNaN(CRIT.x[2])==true) CRIT.x[2]=0;
	double[] A=CRIT.sphereToChart();
	double val=Math.sqrt(A[0]*A[0]+A[1]*A[1]);
	if(Double.isNaN(val)==true) return 0;
	return val;
    }



    
}
