import java.awt.event.*;
import java.awt.*;
import java.math.*;
import java.awt.geom.*;


public class LevelCurve {

    /**This gets the vector at the top of the loop
       level set with value F. Here F(x,y,z)=xy*/
    
    public static Vector topVector(double F) {
	double a=Math.sqrt(F);
	Vector V=new Vector(a,a,Math.sqrt(1-2*a*a));
       return V;
    }

    
    /**This gets one of the two points on 
       the loop level set of value F that lie
       in the  plane Z=0.  The invariant is xy=d*/
    
    public static Vector sideVector(double F) {
       double x=limit(F);
       double y=F/x;
       Vector V=new Vector(x,y,0);
       return V;
    }
    

    public static double limit(double d) {
	double s=Math.sqrt(2);
	double b=Math.sqrt(Math.sqrt(1-4*d*d)+1)/s;
	double c=d/b;
	b=Math.min(b,c);
	return b;
    }


    public static Vector[] curveCheap(int N,double r) {
	Vector[] LIST=new Vector[N];
	for(int i=0;i<N;++i) {
	    double t=2*Math.PI*i/N;
	    double c=Math.cos(t);
	    double s=Math.sin(t);
	    LIST[i]=Vector.chartToSphere(r*c,r*s);
	}
	return LIST;
    }
    
    public static Vector[] curveLongitude(int N,double theta) {
	Vector[] LIST=new Vector[N];
	for(int i=0;i<N;++i) {
	    double ss=1.0*i/(N-1);
	    ss=.5*Math.PI*((1-ss)*(-1)+ss);
	    double x=Math.cos(ss)*Math.cos(theta);
	    double y=Math.cos(ss)*Math.sin(theta);
	    double z=Math.sin(ss);
	    LIST[i]=new Vector(x,y,z);
	}
	return LIST;
    }
    
    public static Vector[] curveLattitude(int N,double psi0) {
	double psi=.5*Math.PI-2*psi0;
	Vector[] LIST=new Vector[N];
	for(int i=0;i<N;++i) {
	    double ss=1.0*i/(N-1);
	    ss=.25*Math.PI+.25*Math.PI*((1-ss)*(-1)+ss);
	    double x=Math.cos(psi)*Math.cos(ss);
	    double y=Math.cos(psi)*Math.sin(ss);
	    double z=Math.sin(psi);
	    LIST[i]=new Vector(x,y,z);
	}
	return LIST;
    }
    
}
