public class PeriodData {

    /**We are no longer using this file.  Originally, we 
       precomputed the holonomy and period length, and
       then used the data as a numerical look-up table for the
       true functions.  Now we have the formulas in terms 
       of elliptic integrals.*/
    

    /**Here is how the data was used*/

    /**Here d ranges from 0 to 1/2.  The value of d
       for the point (a,a,Sqrt(1-2a^2)) is a^2*/

    public static double period(double d) {
	if(d>.49999999) return Math.PI*Math.sqrt(2);
	double[] X=PeriodData.A2();
	double[] Y=PeriodData.PERIOD();
	boolean test=false;
	int count=0;
	while(test==false) {
	    if(d>X[count]) {
		test=true;
		double x0=X[count+0];
		double x1=X[count-1];
		double y0=Y[count+0];
		double y1=Y[count-1];
		double t=(x1-d)/(x1-x0);
		double y=t*y0+(1-t)*y1;
		return y;
	    }
	    if(count==X.length-1) test=true;
	    ++count;
	}
	return(Y[Y.length-1]);
    }


    public static double periodInverse(double d) {
	double[] X=PeriodData.A2();
	double[] Y=PeriodData.PERIOD();
	boolean test=false;
	int count=0;
	if(d<Math.PI*Math.sqrt(2)) return(0.5);
	while(test==false) {
	    if(d<Y[count]) {
		test=true;
		double x0=X[count+0];
		double x1=X[count-1];
		double y0=Y[count+0];
		double y1=Y[count-1];
		double t=(y1-d)/(y1-y0);
		double y=t*x0+(1-t)*x1;
		return y;
	    }
	    if(count==X.length-1) test=true;
	    ++count;
	}
	return(0);
    }



    /**Here is how we generated the period data*/

    public static double periodEuler(double step,Vector U) {	
        double base=Math.PI*Math.sqrt(2);
	if(U.x[0]*U.x[1]>.499999) return base;
	Vector V=new Vector(U);
	int count=0;
	boolean test=false;
	boolean away=false;
	double sign=1;
	while(test==false) {
	    if((away==false)&&(step*count>1)) {
		away=true;
		sign=V.x[2];
	    }
	    if((away==true)&&(sign*V.x[2]<0)) test=true;
	    Vector TANGENT=GroupStructure.structureField(V);
	    Vector dV=TANGENT.scale(step);
	    V=Vector.plus(V,dV);
	    V=V.unit();
	    ++count;
	}
	double period=2*step*count;
	return period;
    }

    
    public static void generateData() {

	System.out.println("start");
	Output OUT=new Output("file");
	double step=0.00000005;
	double N=200;


        OUT.writeln("public class PeriodDataTemp {");
	OUT.writeln("");


	OUT.writeln("public static double[] X() {");
	OUT.write("double[] x={");
	for(int i=0;i<N;++i) {
	    double t=1.0*i/N;
	    double c=Math.cos(Math.PI/4*(1-t));
	    double s=Math.sin(Math.PI/4*(1-t));
	    Double X=new Double(c*s);
	    String S=X.toString();
	    OUT.write(S);
	    if(i<N-1) OUT.write(",");
	}
	OUT.writeln("};");
	OUT.writeln("return x;");
	OUT.writeln("}");


	OUT.writeln("");


	OUT.writeln("public static double[] Y() {");
	OUT.write("double[] y={");
	for(int i=0;i<N;++i) {
	    double t=1.0*i/N;
	    double c=Math.cos(Math.PI/4*(1-t));
	    double s=Math.sin(Math.PI/4*(1-t));
	    Vector V=new Vector(c,s,0);
	    double test=periodEuler(step,V);
	    Double X=new Double(c*s);
	    Double Y=new Double(test);
	    String S=Y.toString();
	    OUT.write(S);
	    if(i<N-1) OUT.write(",");
	    System.out.println("working "+i);
	}
	OUT.writeln("};");
	OUT.writeln("return y;");
	OUT.writeln("}");
	OUT.writeln("");
	OUT.writeln("}");

	System.out.println("done");
    }

    

    public static void generateHolonomyData() {

	System.out.println("start");
	Output OUT=new Output("file");
	double step=0.000001;
	double N=200;


        OUT.writeln("public class PeriodDataTemp {");
	OUT.writeln("");

	double[] X=PeriodData.A2();

	OUT.writeln("public static double[] Z() {");
	OUT.write("double[] z={");
	for(int i=0;i<N;++i) {
	    double val=X[i];
	    double x=Math.sqrt(val);
	    double y=x;
	    double z=Math.sqrt(1-1.999999999*x*x);
	    Vector V=new Vector(x,y,z);
	    double[] data=GeodesicMaker.periodData(step,V);
	    Double D=new Double(data[3]);
	    String S=D.toString();
	    OUT.write(S);
	    if(i<N-1) OUT.write(",");
	    System.out.println("working "+i);
	}
	OUT.writeln("};");
	OUT.writeln("return y;");
	OUT.writeln("}");
	OUT.writeln("");
	OUT.writeln("}");

	System.out.println("done");

    }


    



    

    /**The DATA*/

    /**This is the square of the parameter a.  It corresponds to the vector (A,A,*) having unit length*/
    

public static double[] A2() {
double[] x={0.5,0.4999845788223949,0.49993831624083024,0.4998612151090003,0.49975328018286574,0.49961451812036145,0.499444937480985,0.49924454872526897,0.4990133642141358,0.4987513982081351,0.4984586668665639,0.49813518824647063,0.49778098230154005,0.49739607088086324,0.49698047772758985,0.4965342284774632,0.49605735065723894,0.49554987368298736,0.49501182885827877,0.49444324937225237,0.4938441702975689,0.49321462858824766,0.49255466307738693,0.4918643144747679,0.4911436253643443,0.4903926402016152,0.48961140531088293,0.4887999688823953,0.48795838096937366,0.48708669348492467,0.4861849601988383,0.48525323673427123,0.4842915805643155,0.48330005100845363,0.482278709228899,0.48122761822682364,0.48014684283847153,0.4790364497311596,0.4778965073991651,0.4767270861595006,0.4755282581475768,0.47430009731275224,0.47304267941377265,0.47175608201409686,0.4704403844771127,0.469095667961242,0.46772201541493363,0.46631951157154705,0.4648882429441257,0.46342829782006045,0.4619397662556433,0.4604227400705131,0.4588773128419906,0.45730357989930676,0.4557016383177226,0.45407158691254057,0.4524135262330098,0.45072755855612284,0.44901378788030777,0.44727231991901256,0.44550326209418384,0.4437067235296417,0.4418828150443467,0.44003164914556603,0.4381533400219318,0.43624800353639853,0.43431575721909554,0.43235672026007754,0.4303710135019718,0.4283587594325248,0.4263200821770461,0.42425510749075185,0.4221639627510076,0.42004677694947096,0.4179036806841351,0.4157348061512726,0.41354028713728097,0.4113202590104299,0.40907485871251176,0.40680422475039346,0.40450849718747367,0.40218781763504224,0.39984232924354535,0.397472176693755,0.3950775061878452,0.3926584654403724,0.3902152036691649,0.38774787158611723,0.3852566213878946,0.38274160674654406,0.38020298280001547,0.3776409061425917,0.37505553481522985,0.37244702829581106,0.36981554748930484,0.3671612547178428,0.3644843137107058,0.36178488959422467,0.3590631488815944,0.3563192594626027,0.3535533905932738,0.3507657128854279,0.3479563982961571,0.34512562011721853,0.3422735529643443,0.33940037276647084,0.3365062567548866,0.33359138345229983,0.3306559326618259,0.3277000854558969,0.3247240241650918,0.3217279323668894,0.31871199487434476,0.31567639772468886,0.3126213281678526,0.30954697465491704,0.30645352682648824,0.30334117550099987,0.30021011266294206,0.29706053145101924,0.2938926261462365,0.2907065921599153,0.2875026260216393,0.284280925367132,0.2810416889260653,0.2777851165098011,0.2745114089990659,0.2712207683315594,0.26791339748949833,0.2645895004870953,0.26124928235797445,0.2578929491425237,0.2545207078751856,0.2511327665716862,0.24772933421620372,0.2443106207484774,0.24087683705085758,0.23742819493529727,0.2339649071302867,0.2304871872677312,0.2269952498697734,0.22348931033556055,0.2199695849279576,0.21643629076020698,0.21288964578253636,0.20932986876871407,0.2057571793025544,0.20217179776437252,0.1985739453173903,0.19496384389409413,0.19134171618254486,0.18770778561264154,0.18406227634233896,0.18040541324382087,0.17673742188962854,0.17305852853874645,0.16936896012264566,0.16566894423128548,0.16195870909907467,0.158238483590793,0.1545084971874737,0.15076897997224778,0.14702016261615194,0.14326227636389918,0.13949555301961467,0.13572022493253713,0.13193652498268646,0.1281446865664983,0.12434494358242741,0.12053753041651934,0.11672268192795272,0.11290063343455185,0.1090716206982713,0.10523587991065284,0.10139364767825625,0.09754516100806414,0.0936906572928623,0.08983037429659625,0.08596455013970475,0.08209342328443146,0.07821723252011542,0.07433621694846147,0.07045061596879129,0.06656066926327615,0.06266661678215207,0.05876869872891879,0.05486715554552259,0.05096222789752498,0.0470541566592572,0.04314318289896173,0.039229547863922506,0.03531349296558337,0.031395259764656715,0.027475089956222897,0.023553225354821346,0.01962990787953432,0.01570537953906416,0.011779882416805087,0.007853658655910345,0.0039269504443556705};
return x;
}

    /**Period data*/

public static double[] PERIOD() {
double[] y={4.442882938158366,4.4429345,4.443088599999999,4.4433456,4.4437054,4.4441682,4.4447339,4.4454025999999995,4.4461746,4.447049799999999,4.4480284999999995,4.449110699999999,4.450296799999999,4.4515867,4.4529809,4.454479399999999,4.4560825,4.4577905,4.4596035999999994,4.4615222,4.4635465,4.4656769999999995,4.4679138,4.4702573999999995,4.4727082,4.4752665,4.4779328,4.4807076,4.4835912,4.486584199999999,4.489687,4.4929001,4.4962241999999994,4.4996597,4.503207199999999,4.5068674,4.5106408,4.514527999999999,4.5185298,4.5226467999999995,4.5268798,4.5312294,4.5356964,4.5402816,4.5449858,4.5498098,4.5547545,4.5598206999999995,4.5650093,4.5703214,4.5757577,4.5813194,4.5870074,4.5928227999999995,4.598766599999999,4.604839999999999,4.6110441,4.61738,4.623849,4.6304523,4.6371911,4.6440668,4.6510806,4.658234,4.6655283999999995,4.6729652,4.680546,4.6882722,4.6961454,4.7041672,4.7123393,4.7206635,4.7291414,4.7377747999999995,4.7465657,4.7555157999999995,4.7646272,4.773901899999999,4.7833419,4.7929493999999995,4.8027264,4.8126752999999995,4.8227984,4.833098,4.8435766,4.8542366999999995,4.8650807,4.8761114999999995,4.8873315999999996,4.898744,4.9103515,4.9221569999999994,4.9341636,4.9463745,4.9587927999999994,4.971422,4.984265499999999,4.9973266999999995,5.0106094,5.0241172999999995,5.037854299999999,5.0518244,5.0660316,5.0804804,5.0951749,5.1101199,5.1253199,5.1407798,5.1565046,5.1724995,5.1887697,5.2053208,5.2221585,5.2392886999999995,5.2567175,5.2744513,5.2924964999999995,5.3108601,5.329549,5.3485705,5.367932199999999,5.387641899999999,5.4077079,5.4281387,5.4489429,5.4701298,5.4917089,5.5136902,5.536084,5.558901,5.5821524,5.605849999999999,5.630006,5.6546331,5.679744599999999,5.7053546,5.7314775,5.7581286,5.785324,5.8130804,5.8414154,5.8703474,5.8998959,5.9300814,5.9609252,5.9924501,6.02468,6.05764,6.0913568,6.125858699999999,6.161175399999999,6.1973386999999995,6.234382,6.2723412,6.3112542,6.3511614,6.392106,6.434134299999999,6.4772954,6.5216425,6.5672321,6.6141255,6.6623885,6.7120921,6.7633130999999995,6.8161349,6.8706479,6.926950799999999,6.9851510999999995,7.0453667,7.107726899999999,7.172374199999999,7.2394658,7.3091762,7.3816996999999995,7.457252899999999,7.5360793,7.618453199999999,7.7046852999999995,7.795129599999999,7.890191799999999,7.9903398999999995,8.096117699999999,8.2081618,8.3272248,8.454204599999999,8.590184899999999,8.7364902,8.8947629,9.0670745,9.256089099999999,9.4653139,9.6994981,9.965298599999999,10.2724634,10.6361026,11.0815269,11.6561797,12.466563299999999,13.852495099999999};
return y;
}

    /**Holonomy invariant squared*/

public static double[] HOL2() {
double[] z={9.869595790080739,9.869826823095432,9.87051107336506,9.871653094720195,9.873253072179923,9.875311265059507,9.877823564994651,9.880799265613957,9.884234408790148,9.888129554859855,9.892489781994357,9.897311362707054,9.902599529952518,9.90835515413601,9.914579185639933,9.921277094952181,9.928445555733077,9.936090203169682,9.944212317715282,9.952817702854732,9.961903375243866,9.971479749832088,9.981539588728067,9.992097922452011,10.003152135219079,10.014704143879799,10.026760396214987,10.039327438427918,10.05240748786065,10.066002868737906,10.080120442558563,10.09476275289237,10.109936884409127,10.125645611963035,10.14190067904356,10.158700679462992,10.176053182739667,10.193970306473295,10.212446622015584,10.23149893773088,10.251130945045903,10.271350900151534,10.292162796110127,10.313575197885758,10.335596830493122,10.358236584334037,10.381503520821088,10.405402472718889,10.42994727020739,10.45514312204938,10.48099984069351,10.507531841311529,10.534744948723349,10.562649609054306,10.591260889409142,10.6205896933639,10.650642778762657,10.681435941525235,10.712980847470103,10.7452894365379,10.778378316175734,10.812255620943255,10.846942937288247,10.88245340279816,10.918796113939957,10.955993639896542,10.99406015390077,11.03301020555488,11.072863102696733,11.11363855315933,11.155352320458913,11.198024971680075,11.24167752829813,11.286327131502485,11.332000131832888,11.378714688039572,11.426498204250652,11.475365604787521,11.52534979518745,11.576475599097156,11.62876414905972,11.682245934776674,11.736947811734241,11.792901707427566,11.85013166127588,11.908675512075153,11.968563286959139,12.029830227701028,12.092508187541165,12.15663863320316,12.222251109811591,12.289393493882352,12.358097546370553,12.428413450245671,12.500379714571256,12.574049065781692,12.649458412764583,12.726663309770279,12.805716533238353,12.886668178517258,12.969574327662738,13.054492847923205,13.141483490971936,13.230612271769928,13.321938753684085,13.415537536665772,13.511477015996412,13.609832333426647,13.71068124856533,13.814104293963732,13.920184940862695,14.029014025630866,14.140685677964134,14.255293281528955,14.372942426258222,14.493738401590738,14.617790696524445,14.745221729067454,14.876150197856381,15.010704096702499,15.14902525473434,15.291248582614324,15.437532007200007,15.588031567471079,15.742910318098597,15.902347230953731,16.066525096937685,16.235643713856057,16.409903709419794,16.589524020207538,16.774734229685524,16.965783746668297,17.162925935579015,17.366435823229665,17.576602773789613,17.793739983800613,18.018173214457512,18.250250523875447,18.49034793617958,18.738854461190794,18.9961988975017,19.26283105905,19.539232342476353,19.82592649690635,20.12346164819115,20.43243409531103,20.753483423865116,21.087296314162565,21.43461079848994,21.796233397893396,22.17302382697796,22.565913395952215,22.97592000712974,23.404135225777594,23.8517576382852,24.320086238399178,24.810543958333437,25.324678153194565,25.86419165736167,26.43095140297468,27.02701190301515,27.654633578611058,28.31632510119252,29.01486139619193,29.75332312008719,30.53514633915718,31.364179600267533,32.24472965278659,33.181646665386545,34.18039851806811,35.24719832094817,36.38911200607037,37.61420407823126,38.93175657619374,40.35246068623803,41.88873455879338,43.555065462119636,45.36845412063225,47.34900210450042,49.52064168730089,51.91209633520978,54.5581391343223,57.501276123526154,60.794015602836836,64.50200559301304,68.70840521209074,73.520139674557,79.07705517026085,85.56560915262384,93.24010368804491,102.45656529745952,113.72928585798843,127.82976368240885,145.97038074743966,170.1718873142266,204.0717481135846,254.94513343067422,339.7678856239786,509.468597096304,1018.6943416291452};
return z;
}



}
