import java.awt.event.*;
import java.awt.*;
import java.math.*;
import java.awt.geom.*;


public class GeodesicMaker {

    /**Here is the main routine.
       choice gives the choice of geometries:
          0. SOL
          1. NIL
       step gives the timestep for the numerical approximation. 
       keeping this around .0001 works very well and keeping it
       around .001 works pretty well and is fast.
       length gives the length of the geodesic.
       U gives the Lie algebra vector*/


    public static Vector[] geodesic(int choice,double step,double length,Vector U) {
	int lim=(int)(length/step);
	Vector[] V1=new Vector[lim];
	Vector[] V2=new Vector[lim];
	V1[0]=new Vector(U);
	V2[0]=new Vector(0,0,0);
	for(int i=1;i<lim;++i) {
	    Vector TANGENT=GroupStructure.structureField(choice,V1[i-1]);
	    Vector dW1=TANGENT.scale(step);
	    V1[i]=Vector.plus(V1[i-1],dW1);
	    V1[i]=V1[i].unit();
	    Vector dW2=V1[i].scale(step);
	    V2[i]=GroupStructure.leftTranslation(choice,V2[i-1],dW2);
	}
	return V2;
    }


    /**This gets the endpoint of the geodesic with the given length and initial direction*/
    
    public static Vector geodesicEndpoint(int choice,double step,double length,Vector U) {
	int lim=(int)(length/step);
	Vector V1;
	Vector V2;
	V1=new Vector(U);
	V2=new Vector(0,0,0);
	for(int i=1;i<lim;++i) {
	    Vector TANGENT=GroupStructure.structureField(choice,V1);
	    Vector dW1=TANGENT.scale(step);
	    V1=Vector.plus(V1,dW1);
	    V1=V1.unit();
	    Vector dW2=V1.scale(step);
	    V2=GroupStructure.leftTranslation(choice,V2,dW2);
	}
	return V2;
    }
    
    /**END OF GEODESIC ROUTINES*/



    
    
    /**GETTING VARIOUS INFO ABOUT THE GEODESIC*/
        
    public static double[] periodData(int choice,double step,Vector U) {
	Vector V1;
	Vector V2;
	Vector MEM=new Vector(U);
	V1=new Vector(U);
	V2=new Vector(0,0,0);
	int count=0;
	boolean test=false;
	boolean away=false;
	while(test==false) {
	    if(Vector.dist(MEM,V1)>20*step) away=true;
	    if((away==true)&&(Vector.dist(MEM,V1)<2*step)) test=true;
	    Vector TANGENT=GroupStructure.structureField(choice,V1);
	    Vector dW1=TANGENT.scale(step);
	    V1=Vector.plus(V1,dW1);
	    V1=V1.unit();
	    Vector dW2=V1.scale(step);
	    V2=GroupStructure.leftTranslation(choice,V2,dW2);
	    if(count*step>20) test=true;
	    ++count;
	}
	double x=V2.x[0];
	double y=V2.x[1];
	double n=Math.sqrt(x*x+y*y);
	x=x/n;
	y=y/n;
	double[] data={step*count,x,y,n};
	return data;
    }


    
    
    /**PROJECTING THE GEODESIC INTO THE PLANE**/
    
    /**this controls the projection.*/

    public static double[] turnProjection(double t,double[] data) {
	double[] data2={data[0],0,0,1};
	double c=Math.cos(t);
	double s=Math.sin(t);
	data2[1]=+c*data[1]+s*data[2];
	data2[2]=-s*data[1]+c*data[2];
	return data2;
    }
    

    /**The vector U should be a unit vector*/

    public static Path2D.Double elevationGeodesic(int choice,double proj,double step,double length,Vector U,boolean fit,boolean turns) {
	double[] data=periodData(choice,step,U);
	double length2=length;
	data=turnProjection(proj,data);
	if(turns==true) length2=data[0]*length;
	Vector[] V=geodesic(choice,step,length2,U);
	double[][] g=GeodesicAnalyzer.elevationGeodesic(data[1],data[2],V);
	if(fit==true) g=GeodesicAnalyzer.fitInSquare(g);
	Path2D.Double gp=new Path2D.Double();
	for(int i=0;i<V.length;++i) {
	    if(i==0) gp.moveTo(g[i][0],g[i][1]);
	    if(i!=0) gp.lineTo(g[i][0],g[i][1]);
	}
	return gp;
    }
    
    public static Path2D.Double planGeodesic(int choice,double step,double length,Vector U,boolean turns) {
	double[] data=periodData(choice,step,U);
	double length2=length;
	if(turns==true)  length2=data[0]*length;
	Vector[] V=geodesic(choice,step,length2,U);
	double[][] g=GeodesicAnalyzer.planGeodesic(data[2],data[3],V);
	Path2D.Double gp=new Path2D.Double();
	for(int i=0;i<V.length;++i) {
	    if(i==0) gp.moveTo(g[i][0],g[i][1]);
	    if(i!=0) gp.lineTo(g[i][0],g[i][1]);
	}
	return gp;
    }
    


    
}
